package com.aidlab.aidlab_sdk

import com.aidlab.sdk.communication.*
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger

class AidlabSynchronizationHandler(
        binaryMessenger: BinaryMessenger
): AidlabSynchronizationDelegate {
    
    private var listener: AidlabFlutter.PlatformAidlabSynchronizationDelegate? = null
    
    init {
        listener = AidlabFlutter.PlatformAidlabSynchronizationDelegate(binaryMessenger)
    }

    fun release() {
        listener = null
    }

    override fun didDetectPastUserEvent(timestamp: Long) {
        listener?.didDetectPastUserEvent(AidlabFlutter.UserEvent().apply {
            this.timestamp = timestamp
        }) {}
    }

    override fun didReceivePastAccelerometer(aidlab: IAidlab, timestamp: Long, ax: Float, ay: Float, az: Float) {
        listener?.didReceivePastAccelerometer(AidlabFlutter.VectorEvent().apply {
            this.timestamp = timestamp
            this.x = ax.toDouble()
            this.y = ay.toDouble()
            this.z = az.toDouble()
        }) {}
    }

    override fun didReceivePastActivity(aidlab: IAidlab, timestamp: Long, activity: ActivityType) {
        listener?.didReceivePastActivity(AidlabFlutter.ActivityEvent().apply {
            this.timestamp = timestamp
            this.activityType = activity.value.toLong()
        }) {}
    }

    override fun didReceivePastBodyPosition(aidlab: IAidlab, timestamp: Long, bodyPosition: BodyPosition) {
        listener?.didReceivePastBodyPosition(AidlabFlutter.BodyPositionEvent().apply {
            this.timestamp = timestamp
            this.bodyPosition = bodyPosition.value.toLong()
        }) {}
    }

    override fun didReceivePastECG(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastECG(AidlabFlutter.EcgEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) { index -> values[index].toDouble() }
        }) {}
    }

    override fun didReceivePastGyroscope(aidlab: IAidlab, timestamp: Long, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastGyroscope(AidlabFlutter.VectorEvent().apply {
            this.timestamp = timestamp
            this.x = qx.toDouble()
            this.y = qy.toDouble()
            this.z = qz.toDouble()
        }) {}
    }

    override fun didReceivePastHeartRate(aidlab: IAidlab, timestamp: Long, heartRate: Int) {
        listener?.didReceivePastHeartRate(AidlabFlutter.HeartRateEvent().apply {
            this.timestamp = timestamp
            this.heartRate = heartRate.toLong()
        }) {}
    }

    override fun didReceivePastHrv(aidlab: IAidlab, timestamp: Long, hrv: Int) {
        listener?.didReceivePastHrv(AidlabFlutter.HrvEvent().apply {
            this.timestamp = timestamp
            this.hrv = hrv.toLong()
        }) {}
    }

    override fun didReceivePastMagnetometer(aidlab: IAidlab, timestamp: Long, mx: Float, my: Float, mz: Float) {
        listener?.didReceivePastMagnetometer(AidlabFlutter.VectorEvent().apply {
            this.timestamp = timestamp
            this.x = mx.toDouble()
            this.y = my.toDouble()
            this.z = mz.toDouble()
        }) {}
    }

    override fun didReceivePastOrientation(aidlab: IAidlab, timestamp: Long, roll: Float, pitch: Float, yaw: Float) {
        listener?.didReceivePastOrientation(AidlabFlutter.OrientationEvent().apply {
            this.timestamp = timestamp
            this.roll = roll.toDouble()
            this.pitch = pitch.toDouble()
            this.yaw = yaw.toDouble()
        }) {}
    }

    override fun didReceivePastPressure(aidlab: IAidlab, timestamp: Long, values: IntArray) {
        listener?.didReceivePastPressure(AidlabFlutter.PressureEvent().apply {
            this.timestamp = timestamp
            this.values = values
        }) {}
    }

    override fun didReceivePastQuaternion(aidlab: IAidlab, timestamp: Long, qw: Float, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastQuaternion(AidlabFlutter.QuaternionEvent().apply {
            this.timestamp = timestamp
            this.qw = qw.toDouble()
            this.qx = qx.toDouble()
            this.qy = qy.toDouble()
            this.qz = qz.toDouble()
        }) {}
    }

    override fun didReceivePastRespiration(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastRespiration(AidlabFlutter.RespirationEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) {index -> values[index].toDouble()}
        }) {}
    }

    override fun didReceivePastRespirationRate(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastRespirationRate(AidlabFlutter.RespirationRateEvent().apply {
            this.timestamp = timestamp
            this.value = value.toLong()
        }) {}
    }

    override fun didReceivePastSkinTemperature(aidlab: IAidlab, timestamp: Long, value: Float) {
        listener?.didReceivePastSkinTemperature(AidlabFlutter.SkinTemperatureEvent().apply {
            this.timestamp = timestamp
            this.value = value.toDouble()
        }) {}
    }

    override fun didReceivePastSoundFeatures(aidlab: IAidlab, values: FloatArray) {
        listener?.didReceivePastSoundFeatures(AidlabFlutter.SoundFeaturesEvent().apply {
            this.values = values.map { it.toDouble() }.toDoubleArray()
        }) {}
    }

    override fun didReceivePastSoundVolume(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastSoundVolume(AidlabFlutter.SoundVolumeEvent().apply {
            this.timestamp = timestamp
            this.soundVolume = value.toLong()
        }) {}
    }

    override fun didReceivePastSteps(aidlab: IAidlab, timestamp: Long, value: Long) {
        listener?.didReceivePastSteps(AidlabFlutter.StepsEvent().apply {
            this.timestamp = timestamp
            this.value = value
        }) {}
    }

    override fun didReceiveUnsynchronizedSize(aidlab: IAidlab, unsynchronizedSize: Int, syncBytesPerSecond: Float) {
        listener?.didReceiveUnsynchronizedSize(AidlabFlutter.UnsynchronizedSizeEvent().apply {
            this.unsynchronizedSize = unsynchronizedSize.toLong()
            this.syncBytesPerSecond = syncBytesPerSecond.toDouble()
        }) {}
    }

    override fun syncStateDidChange(aidlab: IAidlab, state: SyncState) {
        Log.d("AIDLAB_FLUTTER", "syncStateDidChange $state")
        listener?.syncStateDidChange(AidlabFlutter.SyncStateEvent().apply {
            this.state = state.value.toLong()
        }) {}
    }
}