import 'dart:ui';

import 'package:flutter/material.dart';

class LineChart extends StatelessWidget {
  final List<double> points;

  LineChart(this.points);

  @override
  Widget build(BuildContext context) {
    return Container(
      child: CustomPaint(painter: ChartPainter(points)),
    );
  }
}

class ChartPainter extends CustomPainter {
  final List<double> points;
  final Paint linePaint = Paint()
    ..color = Colors.red
    ..strokeCap = StrokeCap.round
    ..style = PaintingStyle.fill
    ..strokeWidth = 2;

  ChartPainter(this.points);

  @override
  void paint(Canvas canvas, Size size) {
    if (points.length < 2) return;

    double maxY = points.first;
    double minY = points.first;
    points.forEach((element) {
      if (element > maxY) {
        maxY = element;
      }
      if (element < minY) {
        minY = element;
      }
    });

    final yDiff = maxY - minY;
    if (yDiff == 0) return;

    double dist = size.width / points.length;

    final convertedPoints = List<Offset>.generate(points.length, (index) {
      final sY = size.height - size.height * ((points[index] - minY) / yDiff);
      return Offset(index * dist, sY);
    });

    canvas.drawPoints(PointMode.polygon, convertedPoints, linePaint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
