//
//  AidlabSynchronizationHandler.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//

import Flutter
import UIKit
import AidlabSDK

public class AidlabSynchronizationHandler: AidlabSynchronizationDelegate {

    private var listener: PlatformAidlabSynchronizationDelegate?
    
    func initial(binding: FlutterBinaryMessenger) {
        listener = PlatformAidlabSynchronizationDelegate(binaryMessenger: binding)
    }
    
    func release() {
        listener = nil
    }
    
    public func syncStateDidChange(_ aidlab: IAidlab, state: SyncState) {
        listener?.syncStateDidChange(SyncStateEvent().apply(closure: {
            $0.state = NSNumber(value: state.rawValue)
        }), completion: { _ in })
    }
    
    public func didReceivePastECG(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        listener?.didReceivePastECG(EcgEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.values = FlutterStandardTypedData.init(float64: Data(values: values.map { Double($0) }))
        }), completion: { _ in })
    }
    
    public func didReceivePastRespiration(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        listener?.didReceivePastRespiration(RespirationEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.values = FlutterStandardTypedData.init(float64: Data(values: values.map { Double($0) }))
        }), completion: { _ in })
    }
    
    public func didReceivePastSkinTemperature(_ aidlab: IAidlab, timestamp: UInt64, value: Float) {
        listener?.didReceivePastSkinTemperature(SkinTemperatureEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
    
    public func didReceivePastHeartRate(_ aidlab: IAidlab, timestamp: UInt64, heartRate: Int32) {
        listener?.didReceivePastHeartRate(HeartRateEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.heartRate = NSNumber(value: heartRate)
        }), completion: { _ in })
    }
    
    public func didReceivePastHrv(_ aidlab: IAidlab, timestamp: UInt64, hrv: Int32) {
        listener?.didReceivePastHrv(HrvEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.hrv = NSNumber(value: hrv)
        }), completion: { _ in })
    }

    public func didReceiveUnsynchronizedSize(_ aidlab: IAidlab, unsynchronizedSize: UInt32, syncBytesPerSecond: Float) {
        listener?.didReceiveUnsynchronizedSize(UnsynchronizedSizeEvent().apply(closure: {
            $0.unsynchronizedSize = NSNumber(value: unsynchronizedSize)
            $0.syncBytesPerSecond = NSNumber(value: syncBytesPerSecond)
        }), completion: { _ in })
    }
    
    public func didReceivePastRespirationRate(_ aidlab: IAidlab, timestamp: UInt64, value: UInt32) {
        listener?.didReceivePastRespirationRate(RespirationRateEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
    
    public func didReceivePastActivity(_ aidlab: IAidlab, timestamp: UInt64, activity: ActivityType) {
        listener?.didReceivePastActivity(ActivityEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.activityType = NSNumber(value: activity.rawValue)
        }), completion: { _ in })
    }
    
    public func didReceivePastSteps(_ aidlab: IAidlab, timestamp: UInt64, value: UInt64) {
        listener?.didReceivePastSteps(StepsEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
    
    public func didReceivePastSoundVolume(_ aidlab: IAidlab, timestamp: UInt64, soundVolume: UInt16) {
        listener?.didReceivePastSoundVolume(SoundVolumeEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.soundVolume = NSNumber(value: soundVolume)
        }), completion: { _ in })
    }
    
    public func didReceivePastAccelerometer(_ aidlab: IAidlab, timestamp: UInt64, ax: Float, ay: Float, az: Float) {
        listener?.didReceivePastAccelerometer(VectorEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.x = NSNumber(value: ax)
            $0.y = NSNumber(value: ay)
            $0.z = NSNumber(value: az)
        }), completion: { _ in })
    }
    
    public func didReceivePastGyroscope(_ aidlab: IAidlab, timestamp: UInt64, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastGyroscope(VectorEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.x = NSNumber(value: qx)
            $0.y = NSNumber(value: qy)
            $0.z = NSNumber(value: qz)
        }), completion: { _ in })
    }
    
    public func didReceivePastMagnetometer(_ aidlab: IAidlab, timestamp: UInt64, mx: Float, my: Float, mz: Float) {
        listener?.didReceivePastMagnetometer(VectorEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.x = NSNumber(value: mx)
            $0.y = NSNumber(value: my)
            $0.z = NSNumber(value: mz)
        }), completion: { _ in })
    }
    
    public func didReceivePastQuaternion(_ aidlab: IAidlab, timestamp: UInt64, qw: Float, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastQuaternion(QuaternionEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.qw = NSNumber(value: qw)
            $0.qx = NSNumber(value: qx)
            $0.qy = NSNumber(value: qy)
            $0.qz = NSNumber(value: qz)
        }), completion: { _ in })
    }
    
    public func didReceivePastOrientation(_ aidlab: IAidlab, timestamp: UInt64, roll: Float, pitch: Float, yaw: Float) {
        listener?.didReceivePastOrientation(OrientationEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.roll = NSNumber(value: roll)
            $0.pitch = NSNumber(value: pitch)
            $0.yaw = NSNumber(value: yaw)
        }), completion: { _ in })
    }
    
    public func didReceivePastBodyPosition(_ aidlab: IAidlab, timestamp: UInt64, bodyPosition: BodyPosition) {
        listener?.didReceivePastBodyPosition(BodyPositionEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.bodyPosition = NSNumber(value: bodyPosition.rawValue)
        }), completion: { _ in })
    }
    
    public func didReceivePastPressure(_ aidlab: IAidlab, timestamp: UInt64, values: [Int32]) {
        listener?.didReceivePastPressure(PressureEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.values = FlutterStandardTypedData.init(int32: Data(values: values))
        }), completion: { _ in })
    }
    
    public func didReceivePastSoundFeatures(_ aidlab: IAidlab, values: [Float]) {
        listener?.didReceivePastSoundFeatures(SoundFeaturesEvent().apply(closure: {
            $0.values = FlutterStandardTypedData.init(float64: Data(values: values))
        }), completion: { _ in })
    }
    
    public func didDetectPastUserEvent(_ aidlab: IAidlab, timestamp: UInt64) {
        listener?.didDetectPastUserEvent(UserEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
        }), completion: { _ in })
    }
}
