//
//  SwiftAidlabSdkPlugin.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//

import Flutter
import UIKit

public class SwiftAidlabSdkPlugin: NSObject, FlutterPlugin {
    
    private var aidlabSynchronizationHandler = AidlabSynchronizationHandler()
    private lazy var aidlabHandler = AidlabHandler(synchronizationDelegate: aidlabSynchronizationHandler)
    private lazy var aidlabBluetoothManager = AidlabBluetoothManager()
    private lazy var aidlabSDKHandler = AidlabSdkHandler(aidlabHandler: aidlabHandler, aidlabBluetoothManager: aidlabBluetoothManager)
    
    private static var shared: SwiftAidlabSdkPlugin = SwiftAidlabSdkPlugin()
    
    public static func register(with registrar: FlutterPluginRegistrar) {

        let channel = FlutterMethodChannel(name: "aidlab_sdk", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(SwiftAidlabSdkPlugin.shared, channel: channel)
        
        SwiftAidlabSdkPlugin.shared.aidlabBluetoothManager.initial()
        SwiftAidlabSdkPlugin.shared.aidlabHandler.initial(binding: registrar.messenger())
        SwiftAidlabSdkPlugin.shared.aidlabSynchronizationHandler.initial(binding: registrar.messenger())
        SwiftAidlabSdkPlugin.shared.aidlabSDKHandler.initial(binding: registrar.messenger())
    }
    
    public func detachFromEngine(for registrar: FlutterPluginRegistrar) {
        SwiftAidlabSdkPlugin.shared.aidlabHandler.release()
        SwiftAidlabSdkPlugin.shared.aidlabSDKHandler.release()
        SwiftAidlabSdkPlugin.shared.aidlabSynchronizationHandler.release()
    }
}
