import '../wrappers/aidlab_synchronization_wrapper.dart';
import '../wrappers/platform_aidlab_sdk.dart';

import 'aidlab_synchronization_delegate.dart';
import 'ecg_filtration_method.dart';
import 'internal/internal_aidlab.dart';

class Aidlab {
  final PlatformAidlab _platformAidlab;
  late AidlabSynchronizationWrapper _synchronizationWrapper;
  late InternalAidlab _internalAidlab;

  final String firmwareRevision;
  final String hardwareRevision;
  final String serialNumber;
  final String address;

  InternalAidlab get internalAidlab => _internalAidlab;

  set aidlabSynchronizationDelegate(AidlabSynchronizationDelegate delegate) {
    _synchronizationWrapper.delegate = delegate;
  }

  Aidlab(this.firmwareRevision, this.hardwareRevision, this.serialNumber,
      this.address)
      : _platformAidlab = PlatformAidlab() {
    _synchronizationWrapper = AidlabSynchronizationWrapper(this);
    _internalAidlab = InternalAidlab(_platformAidlab, this);
  }

  void disconnect() {
    _platformAidlab.disconnect();
  }

  void send(String command) {
    _platformAidlab.send(SendRequest()..command = command);
  }

  void setEcgFiltrationMethod(EcgFiltrationMethod ecgFiltrationMethod) {
    _platformAidlab.setECGFiltrationMethod(
        EcgFiltrationRequest()..method = ecgFiltrationMethod.value);
  }

  void startSynchronization() {
    _platformAidlab.startSynchronization();
  }

  void stopSynchronization() {
    _platformAidlab.stopSynchronization();
  }
}
