import 'dart:developer';
import 'dart:typed_data';

import 'package:aidlab_sdk/src/api/internal/internal_aidlab_logger_delegate.dart';
import 'package:aidlab_sdk/src/wrappers/aidlab_logger_delegate_wrapper.dart';
import 'package:flutter/material.dart';

import '../wrappers/aidlab_sdk_wrapper.dart';
import '../wrappers/aidlab_delegate_wrapper.dart';

import '../wrappers/platform_aidlab_sdk.dart';
import 'aidlab_delegate.dart';
import 'aidlab_sdk_delegate.dart';
import 'signal.dart';
import 'scan_mode.dart';

class AidlabSdk {
  AidlabSdk._inner();

  static AidlabSdk instance = AidlabSdk._inner();

  final PlatformAidlabSdk _platformSdk = PlatformAidlabSdk();
  bool _initialized = false;

  set aidlabSdkDelegate(AidlabSdkDelegate? aidlabSdkDelegate) {
    if (aidlabSdkDelegate != null) {
      AidlabSdkDelegateWrapper.listen(aidlabSdkDelegate);
    } else {
      AidlabSdkDelegateWrapper.release();
    }
  }

  set internalAidlabLoggerDelegate(InternalAidlabLoggerDelegate? delegate) {
    if (delegate != null) {
      AidlabLoggerDelegateWrapper.listen(delegate);
    } else {
      AidlabLoggerDelegateWrapper.release();
    }
  }

  Future init() async {
    WidgetsFlutterBinding.ensureInitialized();
    await _platformSdk.init();
    _initialized = true;
  }

  void connect(
      String address, List<Signal> signals, AidlabDelegate aidlabDelegate) {
    _assureInitialized();

    log("Connecting");
    AidlabDelegateWrapper.listen(aidlabDelegate);
    _platformSdk.connect(ConnectRequest()
      ..address = address
      ..signals = Int32List.fromList(signals.map((e) => e.value).toList()));
  }

  void scanForDevices(ScanMode scanMode) {
    _assureInitialized();
    _platformSdk.scanForDevices(ScanModeRequest()..mode = scanMode.value);
  }

  void stopDeviceScan() {
    _assureInitialized();
    _platformSdk.stopDeviceScan();
  }

  void _assureInitialized() {
    if (!_initialized) {
      throw StateError(
          "AidlabSDK not initialized, call init() before using it.");
    }
  }
}
