import 'package:aidlab_sdk/aidlab_sdk.dart';
import 'platform_aidlab_sdk.dart';

class AidlabDelegateWrapper implements PlatformAidlabDelegate {
  final AidlabDelegate delegate;

  Aidlab? _aidlab;

  AidlabDelegateWrapper._inner(this.delegate);

  static void listen(AidlabDelegate delegate) {
    final wrapper = AidlabDelegateWrapper._inner(delegate);
    PlatformAidlabDelegate.setup(wrapper);
  }

  static void release() {
    PlatformAidlabDelegate.setup(null);
  }

  @override
  void didConnect(ConnectEvent arg) {
    _aidlab = Aidlab(arg.firmwareRevision!, arg.hardwareRevision!,
        arg.serialNumber!, arg.address!);
    delegate.didConnectAidlab(_aidlab!);
  }

  @override
  void didDetectExercise(ExerciseEvent arg) {
    delegate.didDetectExercise(
        _aidlab!, ExerciseExtension.fromValue(arg.exercise!));
  }

  @override
  void didDisconnect(DisconnectEvent arg) {
    delegate.didDisconnectAidlab(
        _aidlab, DisconnectReasonExtension.fromValue(arg.disconnectReason!));
  }

  @override
  void didReceiveAccelerometer(VectorEvent arg) {
    delegate.didReceiveAccelerometer(
        _aidlab!, arg.timestamp!, arg.x!, arg.y!, arg.z!);
  }

  @override
  void didReceiveActivity(ActivityEvent arg) {
    delegate.didReceiveActivity(_aidlab!, arg.timestamp!,
        ActivityTypeExtension.fromValue(arg.activityType!));
  }

  @override
  void didReceiveBatteryLevel(BatteryLevelEvent arg) {
    // Battery is from different characteristics than other signal
    // on iOS can be received before Aidlab is fully connected
    if (_aidlab == null) return;
    delegate.didReceiveBatteryLevel(_aidlab!, arg.stateOfCharge!);
  }

  @override
  void didReceiveCommand(EmptyEvent arg) {
    delegate.didReceiveCommand(_aidlab!);
  }

  @override
  void didReceiveEcg(EcgEvent arg) {
    delegate.didReceiveECG(_aidlab!, arg.timestamp!, arg.values!.toList());
  }

  @override
  void didReceiveError(ErrorEvent arg) {
    delegate.didReceiveError(arg.error!);
  }

  @override
  void didReceiveGyroscope(VectorEvent arg) {
    delegate.didReceiveGyroscope(
        _aidlab!, arg.timestamp!, arg.x!, arg.y!, arg.z!);
  }

  @override
  void didReceiveHeartRate(HeartRateEvent arg) {
    delegate.didReceiveHeartRate(_aidlab!, arg.timestamp!, arg.heartRate!);
  }

  @override
  void didReceiveMagnetometer(VectorEvent arg) {
    delegate.didReceiveMagnetometer(
        _aidlab!, arg.timestamp!, arg.x!, arg.y!, arg.z!);
  }

  @override
  void didReceiveMessage(MessageEvent arg) {
    delegate.didReceiveMessage(_aidlab!, arg.process!, arg.message!);
  }

  @override
  void didReceiveOrientation(OrientationEvent arg) {
    delegate.didReceiveOrientation(
        _aidlab!, arg.timestamp!, arg.roll!, arg.pitch!, arg.yaw!);
  }

  @override
  void didReceiveQuaternion(QuaternionEvent arg) {
    delegate.didReceiveQuaternion(
        _aidlab!, arg.timestamp!, arg.qw!, arg.qx!, arg.qy!, arg.qz!);
  }

  @override
  void didReceiveRespiration(RespirationEvent arg) {
    delegate.didReceiveRespiration(
        _aidlab!, arg.timestamp!, arg.values!.toList());
  }

  @override
  void didReceiveRespirationRate(RespirationRateEvent arg) {
    delegate.didReceiveRespirationRate(_aidlab!, arg.timestamp!, arg.value!);
  }

  @override
  void didReceiveSkinTemperature(SkinTemperatureEvent arg) {
    delegate.didReceiveSkinTemperature(_aidlab!, arg.timestamp!, arg.value!);
  }

  @override
  void didReceiveSoundVolume(SoundVolumeEvent arg) {
    delegate.didReceiveSoundVolume(_aidlab!, arg.timestamp!, arg.soundVolume!);
  }

  @override
  void didReceiveSteps(StepsEvent arg) {
    delegate.didReceiveSteps(_aidlab!, arg.timestamp!, arg.value!);
  }

  @override
  void wearStateDidChange(WearStateEvent arg) {
    delegate.wearStateDidChange(
        _aidlab!, WearStateExtension.fromValue(arg.wearState!));
  }

  @override
  void didDetectUserEvent(UserEvent arg) {
    delegate.didDetectUserEvent(_aidlab!, arg.timestamp!);
  }

  @override
  void didReceiveBodyPosition(BodyPositionEvent arg) {
    delegate.didReceiveBodyPosition(_aidlab!, arg.timestamp!,
        BodyPositionExtension.fromValue(arg.bodyPosition!));
  }

  @override
  void didReceiveHrv(HrvEvent arg) {
    delegate.didReceiveHrv(_aidlab!, arg.timestamp!, arg.hrv!);
  }

  @override
  void didReceiveSoundFeatures(SoundFeaturesEvent arg) {
    delegate.didReceiveSoundFeatures(_aidlab!, arg.values!);
  }
}
